@@ -97,8 +97,7 @@ static void      window_menu_plugin_set_property            (GObject
                                                              guint               prop_id,
                                                              const GValue       *value,
                                                              GParamSpec         *pspec);
-static void      window_menu_plugin_style_set               (GtkWidget          *widget,
-                                                             GtkStyle           *previous_style);
+static void      window_menu_plugin_style_updated           (GtkWidget          *widget);
 static void      window_menu_plugin_screen_changed          (GtkWidget          *widget,
                                                              GdkScreen          *previous_screen);
 static void      window_menu_plugin_construct               (XfcePanelPlugin    *panel_plugin);
@@ -153,7 +152,7 @@ window_menu_plugin_class_init (WindowMenuPluginClass *klass)
   gobject_class->set_property = window_menu_plugin_set_property;
 
   gtkwidget_class = GTK_WIDGET_CLASS (klass);
-  gtkwidget_class->style_set = window_menu_plugin_style_set;
+  gtkwidget_class->style_updated = window_menu_plugin_style_updated;
 
   plugin_class = XFCE_PANEL_PLUGIN_CLASS (klass);
   plugin_class->construct = window_menu_plugin_construct;
@@ -372,13 +371,12 @@ window_menu_plugin_set_property (GObject      *object,
 
 
 static void
-window_menu_plugin_style_set (GtkWidget *widget,
-                              GtkStyle  *previous_style)
+window_menu_plugin_style_updated (GtkWidget *widget)
 {
   WindowMenuPlugin *plugin = XFCE_WINDOW_MENU_PLUGIN (widget);
 
   /* let gtk update the widget style */
-  (*GTK_WIDGET_CLASS (window_menu_plugin_parent_class)->style_set) (widget, previous_style);
+  (*GTK_WIDGET_CLASS (window_menu_plugin_parent_class)->style_updated) (widget);
 
   /* read the style properties */
   gtk_widget_style_get (GTK_WIDGET (plugin),
@@ -401,7 +399,7 @@ window_menu_plugin_screen_changed (GtkWidget *widget,
   /* get the wnck screen */
   screen = gtk_widget_get_screen (widget);
   panel_return_if_fail (GDK_IS_SCREEN (screen));
-  wnck_screen = wnck_screen_get (panel_screen_get_number (screen));
+  wnck_screen = panel_wnck_screen_get (panel_screen_get_number (screen));
   panel_return_if_fail (WNCK_IS_SCREEN (wnck_screen));
 
   /* leave when we same wnck screen was picked */
@@ -626,7 +624,7 @@ window_menu_plugin_set_icon (WindowMenuPlugin *plugin,
                              WnckWindow       *window)
 {
   GdkPixbuf *pixbuf;
-  gint       icon_size;
+  gint       icon_size, scale_factor;
 
   panel_return_if_fail (XFCE_IS_WINDOW_MENU_PLUGIN (plugin));
   panel_return_if_fail (WNCK_IS_WINDOW (window));
@@ -637,13 +635,18 @@ window_menu_plugin_set_icon (WindowMenuPlugin *plugin,
   gtk_widget_set_tooltip_text (plugin->icon, wnck_window_get_name (window));
 
   icon_size = xfce_panel_plugin_get_icon_size (XFCE_PANEL_PLUGIN (plugin));
-  if (icon_size <= 31)
+  scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (plugin));
+  if (icon_size * scale_factor <= 31)
     pixbuf = wnck_window_get_mini_icon (window);
   else
     pixbuf = wnck_window_get_icon (window);
 
   if (G_LIKELY (pixbuf != NULL))
-    gtk_image_set_from_pixbuf (GTK_IMAGE (plugin->icon), pixbuf);
+    {
+      cairo_surface_t *surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+      gtk_image_set_from_surface (GTK_IMAGE (plugin->icon), surface);
+      cairo_surface_destroy (surface);
+    }
   else
     {
       gtk_image_set_from_icon_name (GTK_IMAGE (plugin->icon), "image-missing", icon_size);
@@ -1006,6 +1009,7 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
   gchar       *decorated = NULL;
   GtkWidget   *mi, *label, *image;
   GdkPixbuf   *pixbuf, *lucent = NULL, *scaled = NULL;
+  gint         scale_factor;
 
   panel_return_val_if_fail (WNCK_IS_WINDOW (window), NULL);
 
@@ -1058,6 +1062,9 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
     {
       /* get the window icon */
       pixbuf = wnck_window_get_mini_icon (window);
+      scale_factor = gtk_widget_get_scale_factor (GTK_WIDGET (plugin));
+      icon_w *= scale_factor;
+      icon_h *= scale_factor;
       if (pixbuf != NULL
           && (gdk_pixbuf_get_width (pixbuf) < icon_w
               || gdk_pixbuf_get_height (pixbuf) < icon_h))
@@ -1065,6 +1072,8 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
 
       if (pixbuf != NULL)
         {
+          cairo_surface_t *surface;
+
           /* scale the icon if needed */
           if (gdk_pixbuf_get_width (pixbuf) > icon_w
               || gdk_pixbuf_get_height (pixbuf) > icon_h)
@@ -1084,7 +1093,9 @@ window_menu_plugin_menu_window_item_new (WnckWindow           *window,
             }
 
           /* set the menu item label */
-          image = gtk_image_new_from_pixbuf (pixbuf);
+          surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale_factor, NULL);
+          image = gtk_image_new_from_surface (surface);
+          cairo_surface_destroy (surface);
           panel_image_menu_item_set_image (mi, image);
           gtk_widget_show (image);
 
